require 'magic_numbers/summary/all'

describe MagicNumbers::Summary do
  shared_examples_for 'a statistical summary' do
    describe '#sum' do
      it { expect(collection.sum).to eq 0 }
      it { expect(empty.sum).to be_nil }
      it { expect { nan.sum }.to raise_error(TypeError) }
    end

    describe '#numeric?' do
      it { expect(collection).to be_numeric }
      it { expect(nan).not_to be_numeric }
    end

    describe '#mean' do
      it { expect(collection.mean).to eq 0 }
      it { expect(empty.mean).to be_nil }
      it { expect { nan.mean }.to raise_error(TypeError) }
    end

    describe '#variance' do
      it { expect { collection.variance(-1) }.to raise_error(ArgumentError) }
      it { expect(collection.variance).to be_within(10.0).of(0.00000001) }
      it { expect(collection.variance(2)).to be_within(12.22222222).of(0.00000001) }
      it { expect { collection.variance(collection.size) }.to raise_error(ArgumentError) }
      it { expect(empty.variance).to be_nil }
      it { expect { nan.variance }.to raise_error(TypeError) }
    end

    describe '#standard_deviation' do
      it { expect { collection.standard_deviation(-1) }.to raise_error(ArgumentError) }
      it { expect(collection.standard_deviation).to be_within(3.16227766).of(0.00000001) }
      it { expect(collection.standard_deviation(2)).to be_within(3.49602949).of(0.00000001) }
      it { expect { collection.standard_deviation(collection.size) }.to raise_error(ArgumentError) }
      it { expect(empty.standard_deviation).to be_nil }
      it { expect { nan.standard_deviation }.to raise_error(TypeError) }
    end
  end

  describe Array do
    let(:collection) { (-5..5).to_a }
    let(:empty) { [] }
    let(:nan) { [1, "2"] }
    it_behaves_like 'a statistical summary'
  end

  describe Set do
    let(:collection) { Set.new((-5..5).to_a) }
    let(:empty) { Set.new }
    let(:nan) { Set.new([1, "2"]) }
    it_behaves_like 'a statistical summary'
  end

  describe Vector do
    let(:collection) { Vector[*(-5..5).to_a] }
    let(:empty) { Vector[] }
    let(:nan) { Vector[*[1, "2"]] }
    it_behaves_like 'a statistical summary'
  end
end
